@@ -198,7 +198,7 @@ void Group::MoveToFront (Item *item)
 		break;
 	if (i != end) {
 		m_Children.erase (i);
-		m_Children.push_front (item);
+		m_Children.push_back (item); // back items are displayed at last
 	}
 }
 
@@ -210,7 +210,7 @@ void Group::MoveToBack (Item *item)
 		break;
 	if (i != end) {
 		m_Children.erase (i);
-		m_Children.push_back (item);
+		m_Children.push_front (item); // front items are displayed at first
 	}
 }
 
