@@ -32,6 +32,8 @@
 #include "gcp-stock-pixbufs.h"
 #include "text.h"
 #include "plugin.h"
+#include "mechanism-arrow.h"
+#include "mechanism-step.h"
 #include "mesomer.h"
 #include "mesomery.h"
 #include "mesomery-arrow.h"
@@ -242,6 +244,16 @@ static Object* CreateFragment ()
 	return new Fragment ();
 }
 
+static Object* CreateMechanismArrow ()
+{
+	return new MechanismArrow ();
+}
+
+static Object* CreateMechanismStep ()
+{
+	return new MechanismStep (MechanismStepType);
+}
+
 bool	Application::m_bInit = false;
 bool	Application::m_Have_Ghemical = false;
 bool	Application::m_Have_InChI = false;
@@ -320,6 +332,8 @@ Application::Application ():
 		Object::AddRule ("mesomery", RuleMustContain, "mesomer");
 		Object::AddRule ("mesomery", RuleMustContain, "mesomery-arrow");
 		Object::AddRule ("mesomery-arrow", RuleMustBeIn, "mesomery");
+		MechanismArrowType = Object::AddType ("mechanism-arrow", CreateMechanismArrow);
+		MechanismStepType = Object::AddType ("mechanism-step", CreateMechanismStep);
 
 		// Create global signal ids
 		OnChangedSignal = Object::CreateNewSignalId ();
@@ -332,6 +346,15 @@ Application::Application ():
 		m_Style = gtk_rc_get_style(w);
 		gtk_widget_destroy (w);
 
+		// load settings before plugins
+		m_ConfNode = go_conf_get_node (GetConfDir (), GCP_CONF_DIR_SETTINGS);
+		GCU_GCONF_GET ("compression", int, CompressionLevel, 0)
+		GCU_GCONF_GET ("invert-wedge-hashes", bool, InvertWedgeHashes, false)
+		bool CopyAsText;
+		GCU_GCONF_GET ("copy-as-text", bool, CopyAsText, false)
+		ClipboardFormats = CopyAsText? GCP_CLIPBOARD_ALL: GCP_CLIPBOARD_NO_TEXT;
+		m_NotificationId = go_conf_add_monitor (m_ConfNode, NULL, (GOConfMonitorFunc) on_config_changed, this);
+
 		// load plugins
 		Plugin::LoadPlugins ();
 		m_bInit = true;
@@ -381,13 +404,6 @@ Application::Application ():
 		}
 	}
 	
-	m_ConfNode = go_conf_get_node (GetConfDir (), GCP_CONF_DIR_SETTINGS);
-	GCU_GCONF_GET ("compression", int, CompressionLevel, 0)
-	GCU_GCONF_GET ("tearable-mendeleiev", bool, TearableMendeleiev, false)
-	bool CopyAsText;
-	GCU_GCONF_GET ("copy-as-text", bool, CopyAsText, false)
-	ClipboardFormats = CopyAsText? GCP_CLIPBOARD_ALL: GCP_CLIPBOARD_NO_TEXT;
-	m_NotificationId = go_conf_add_monitor (m_ConfNode, NULL, (GOConfMonitorFunc) on_config_changed, this);
 	// make themes permanent with this as a dummy client
 	list <string> Names = TheThemeManager.GetThemesNames ();
 	list <string>::iterator j, jend = Names.end ();
@@ -1121,8 +1137,8 @@ void Application::AddActions (GtkRadioActionEntry const *entries, int nb, char c
 				// set the pixbuf color to the corresponding style for the style
 				unsigned char red, blue, green;
 				red = m_Style->fg[c].red >> 8;
-				green = m_Style->fg[c].red >> 8;
-				blue = m_Style->fg[c].red >> 8;
+				green = m_Style->fg[c].green >> 8;
+				blue = m_Style->fg[c].blue >> 8;
 				unsigned char *line, *cur;
 				line = gdk_pixbuf_get_pixels (icon);
 				int i, j, rows, cols, rowstride;
@@ -1207,16 +1223,24 @@ void Application::ActivateWindowsActionWidget (const char *path, bool activate)
 	}
 }
 
+void Application::UpdateAllTargets ()
+{
+	set <Target *>::iterator target, tend = m_Targets.end ();
+	for (target = m_Targets.begin (); target != tend; target++) {
+		Document *doc = (*target)->GetDocument ();
+		doc->GetView ()->Update (doc);
+	}
+	// now propagate to tools
+	std::map <std::string, Tool*>::iterator i, iend = m_Tools.end ();
+	for (i = m_Tools.begin (); i != iend; i++)
+		(*i).second->OnConfigChanged ();
+}
+
 void Application::OnConfigChanged (GOConfNode *node, gchar const *name)
 {
 	GCU_UPDATE_KEY ("compression", int, CompressionLevel, {})
-	GCU_UPDATE_KEY ("tearable-mendeleiev", bool, TearableMendeleiev,
-					{
-						Tools *ToolsBox = dynamic_cast<Tools*> (GetDialog ("tools"));
-						if (ToolsBox)
-							ToolsBox->Update ();
-					})
 	bool CopyAsText;
+	GCU_UPDATE_KEY ("invert-wedge-hashes", bool, InvertWedgeHashes, UpdateAllTargets ();)
 	GCU_UPDATE_KEY ("copy-as-text", bool, CopyAsText, ClipboardFormats = CopyAsText?GCP_CLIPBOARD_ALL: GCP_CLIPBOARD_NO_TEXT;)
 }
 
