@@ -246,13 +246,16 @@ void Atom::Update ()
 	int nb, nexplp = 0, nexplu = 0; //nexplp is the number of explicit lone pairs
 	//nexplu is the number of explicit unpaired electrons
 	map<string, Object*>::iterator i;
-	Electron* electron = (Electron*) GetFirstChild (i);
-	while (electron) { 
-		if (electron->IsPair ())
-			nexplp++;
-		else
-			nexplu++;
-		electron = (Electron*) GetNextChild (i);
+	Object *obj = GetFirstChild (i);
+	while (obj) {
+		Electron *electron = dynamic_cast <Electron *> (obj);
+		if (electron) {
+			if (electron->IsPair ())
+				nexplp++;
+			else
+				nexplu++;
+		}
+		obj = GetNextChild (i);
 	}
 	int nbonds = GetTotalBondsNumber ();
 	if (m_Valence > 0 && !m_Element->IsMetallic ()) {
@@ -1257,16 +1260,18 @@ void Atom::SetSelected (int state)
 	std::list<gccv::Item *>::iterator it;
 	gccv::Item *item = group->GetFirstChild (it);
 	while (item) {
-		gccv::FillItem *fill;
-		gccv::Text *text;
-		if ((text = dynamic_cast <gccv::Text *> (item)))
-			text->SetColor (othercolor);
-		else if ((fill = dynamic_cast <gccv::Rectangle *> (item)))
-			fill->SetFillColor (textcolor);
-		else if ((fill = dynamic_cast <gccv::FillItem *> (item)))
-			fill->SetFillColor (othercolor);
-		else
-			static_cast <gccv::LineItem *> (item)->SetLineColor (othercolor);
+		if (item->GetClient () == this) {
+			gccv::FillItem *fill;
+			gccv::Text *text;
+			if ((text = dynamic_cast <gccv::Text *> (item)))
+				text->SetColor (othercolor);
+			else if ((fill = dynamic_cast <gccv::Rectangle *> (item)))
+				fill->SetFillColor (textcolor);
+			else if ((fill = dynamic_cast <gccv::FillItem *> (item)))
+				fill->SetFillColor (othercolor);
+			else
+				static_cast <gccv::LineItem *> (item)->SetLineColor (othercolor);
+		}
 		item = group->GetNextChild (it);
 	}
 }
@@ -1668,8 +1673,24 @@ xmlNodePtr Atom::Save (xmlDocPtr xml) const
 		xmlNewProp (node, (xmlChar*) "show-symbol", (xmlChar*) "true");
 	}
 	if (m_HPosStyle != AUTO_HPOS) {
-		xmlNewProp (node, (xmlChar*) "H-position",
-					(xmlChar*) ((m_HPosStyle == LEFT_HPOS)? "left": "right"));
+		char const *pos;
+		switch (m_HPosStyle) {
+		default:
+		case RIGHT_HPOS:
+			pos = "right";
+			break;
+		case LEFT_HPOS:
+			pos = "left";
+			break;
+		case TOP_HPOS:
+			pos = "top";
+			break;
+		case BOTTOM_HPOS:
+			pos = "bottom";
+			break;
+		}
+		xmlNewProp (node, reinterpret_cast <xmlChar const*> ("H-position"),
+					reinterpret_cast <xmlChar const*> (pos));
 	}
 	return node;
 }
@@ -1687,6 +1708,15 @@ bool Atom::Load (xmlNodePtr node)
 			electron = new Electron (this, false);
 		else if (!strcmp ((char*) child->name, "electron-pair"))
 			electron = new Electron (this, true);
+		else if (strcmp (reinterpret_cast <char const *> (child->name), "position")
+		    && strcmp (reinterpret_cast <char const *> (child->name), "text")) {
+			Object *obj = CreateObject (reinterpret_cast <char const *> (child->name));
+			if (obj) {
+				AddChild (obj);
+				if (!obj->Load (child))
+						return false;
+			}
+		}
 		if (electron && !electron->Load (child))
 			return false;
 		child = child->next;
@@ -1751,6 +1781,12 @@ bool Atom::Load (xmlNodePtr node)
 			m_HPosStyle = LEFT_HPOS;
 		else if (!strcmp (buf, "right"))
 			m_HPosStyle = RIGHT_HPOS;
+		else if (!strcmp (buf, "top"))
+			m_HPosStyle = TOP_HPOS;
+		else if (!strcmp (buf, "bottom"))
+			m_HPosStyle = BOTTOM_HPOS;
+		else // who know?
+			m_HPosStyle = AUTO_HPOS;
 		xmlFree (buf);
 		Update ();
 	}
@@ -1945,6 +1981,7 @@ void Atom::AddItem ()
 	y *= theme->GetZoomFactor ();
 	// always use a group, even if not needed
 	gccv::Group *group = new gccv::Group (view->GetCanvas ()->GetRoot (), x, y, this);
+	view->GetCanvas ()->GetRoot ()->MoveToFront (group);
 	if ((GetZ() != 6) || (GetBondsNumber() == 0) || m_ShowSymbol) {
 		gccv::Text *text = new gccv::Text (group, 0., 0., this);
 		text->SetColor ((view->GetData ()->IsSelected (this))? SelectColor: GO_COLOR_BLACK);
@@ -2037,6 +2074,7 @@ void Atom::AddItem ()
 		else
 			markup = g_strdup (glyph);
 		gccv::Text *text = new gccv::Text (group, x, y, NULL);
+		text->SetColor ((view->GetData ()->IsSelected (this))? SelectColor: GO_COLOR_BLACK);
 		text->SetFillColor (0);
 		text->SetPadding (theme->GetPadding ());
 		text->SetLineColor (0);
