@@ -24,32 +24,33 @@
 
 #include "config.h"
 #include "application.h"
-#include "view.h"
-#include "document.h"
-#include "settings.h"
+#include "atom.h"
+#include "bond.h"
 #include "docprop.h"
+#include "document.h"
 #include "fragment.h"
-#include "reaction.h"
 #include "mesomery.h"
 #include "molecule.h"
+#include "reaction.h"
 #include "residue.h"
+#include "settings.h"
 #include "text.h"
 #include "theme.h"
 #include "tool.h"
+#include "view.h"
+#include "widgetdata.h"
 #include "window.h"
-#include <gcu/objprops.h>
 #include <gcu/loader.h>
-#include <goffice/math/go-rangefunc.h>
-#include <unistd.h>
-#include <libgen.h>
-#include <gtk/gtk.h>
-#include <gio/gio.h>
+#include <gcu/objprops.h>
 #include <glib/gi18n-lib.h>
 #include <openbabel/mol.h>
 #include <openbabel/obconversion.h>
 #include <clocale>
 #include <stack>
 #include <cstring>
+#include <sstream>
+#include <libgen.h>
+#include <unistd.h>
 
 using namespace OpenBabel;
 using namespace std;
@@ -151,7 +152,7 @@ void Document::Clear ()
 
 GtkWidget* Document::GetWidget ()
 {
-	return (m_pView) ? m_pView->GetWidget () : NULL;
+	return (m_pView)? m_pView->GetWidget (): NULL;
 }
 
 
@@ -159,8 +160,9 @@ const gchar* Document::GetTitle () const
 {
 	if (m_title)
 		return m_title;
-	else
-		return GetLabel ();
+	else if (m_label)
+		return m_label;
+	return (m_Window)? m_Window->GetDefaultTitle (): NULL;
 }
 
 const gchar* Document::GetLabel () const
@@ -184,8 +186,8 @@ void Document::SetFileName (string const &Name, const gchar* mime_type)
 	int j = strlen (m_filename) - 1;
 	while ((i < j) && (m_filename[j] != '.'))
 		j--;
-	if (m_label)
-		g_free (m_label);
+	g_free (m_label);
+	m_label = NULL;
 	char *ext = m_filename + j + 1;
 	list<string> &exts = m_pApp->GetExtensions (m_FileType);
 	list<string>::iterator cur, end = exts.end ();
@@ -414,7 +416,7 @@ void Document::ExportOB () const
 	}
 }
 
-void Document::DoPrint (GtkPrintOperation *print, GtkPrintContext *context) const
+void Document::DoPrint (G_GNUC_UNUSED GtkPrintOperation *print, GtkPrintContext *context, G_GNUC_UNUSED int page) const
 {
 	cairo_t *cr;
 	double width, height, x, y, w, h;
@@ -426,9 +428,9 @@ void Document::DoPrint (GtkPrintOperation *print, GtkPrintContext *context) cons
 	// adjust position and size
 	GtkWidget* widget = m_pView->GetWidget ();
 	WidgetData* pData = (WidgetData*) g_object_get_data (G_OBJECT (widget), "data");
-	ArtDRect rect;
+	gccv::Rect rect;
 	pData->GetObjectBounds (this, &rect);
-	double scale = .75;
+	double scale = .75; // FIXME: should we use the real screen resolution?
 	x = rect.x0;
 	y = rect.y0;
 	w = rect.x1 - rect.x0;
@@ -479,7 +481,10 @@ void Document::AddAtom (Atom* pAtom)
 		pAtom->SetId (id);
 		Id = id;
 	}
-	m_pView->AddObject (pAtom);
+	if (!pAtom->GetParent ())
+		AddChild (pAtom);
+	if (m_pView->GetCanvas ())
+		m_pView->AddObject (pAtom);
 	if (m_bIsLoading)
 		return;
 	Molecule* mol = new Molecule ();
@@ -507,10 +512,10 @@ void Document::AddFragment (Fragment* pFragment)
 		pFragment->SetId (id);
 		Id = id;
 	}
+	AddObject(pFragment);
 	m_pView->AddObject (pFragment);
 	if (m_bIsLoading)
 		return;
-//	AddObject(pFragment);
 	if (!pFragment->GetMolecule ()) {
 		Molecule* mol = new Molecule ();
 		i = 1;
@@ -538,11 +543,14 @@ void Document::AddBond (Bond* pBond)
 		while (GetDescendant(id) != NULL);
 		pBond->SetId (id);
 	}
-	AddChild (pBond);
+	if (pBond->GetParent () == NULL)
+		AddChild (pBond);
 	Atom *pAtom0 = (Atom*) pBond->GetAtom (0), *pAtom1 = (Atom*) pBond->GetAtom (1);
-	m_pView->Update (pAtom0);
-	m_pView->Update (pAtom1);
-	m_pView->AddObject (pBond);
+	if (m_pView->GetCanvas ()) {
+		pAtom0->UpdateItem ();
+		pAtom1->UpdateItem ();
+		pBond->AddItem ();
+	}
 	if (m_bIsLoading)
 		return;
 	//search molecules
@@ -605,6 +613,8 @@ void Document::Save () const
 	setlocale (LC_TIME, "C");
 
 	try {
+		if (m_pApp && m_pApp->Save (m_filename, m_FileType.c_str (), this, ContentType2D))
+			return;
 		if (m_FileType != "application/x-gchempaint")
 			ExportOB ();
 		else {
@@ -759,12 +769,13 @@ bool Document::Load (xmlNodePtr root)
 		pObject = CreateObject ((const char*) child->name, this);
 		if (pObject) {
 			if (!pObject->Load (child))
-				delete pObject;
+				Remove (pObject);
 			else
 				m_pView->AddObject (pObject);
 		}
 		node = node->next;
 	}
+	Loaded ();
 	m_pView->Update (this);
 	Update ();
 	m_Empty = !HasChildren ();
@@ -1215,6 +1226,7 @@ void Document::LoadObjects (xmlNodePtr node)
 		else delete pBond;
 		child = GetNextNodeByName (child->next, "bond");
 	}
+	Loaded ();
 }
 
 Operation* Document::GetNewOperation (OperationType type)
@@ -1256,6 +1268,7 @@ void Document::AddData (xmlNodePtr node)
 	}
 	m_bIsLoading = false;
 	EmptyTranslationTable ();
+	Loaded ();
 	FinishOperation ();
 }
 
@@ -1286,7 +1299,7 @@ double Document::GetYAlign ()
 		return Child->GetYAlign ();
 	} else {
 		WidgetData* pData = (WidgetData*) g_object_get_data (G_OBJECT (GetWidget ()), "data");
-		ArtDRect rect;
+		gccv::Rect rect;
 		pData->GetObjectBounds (this, &rect);
 		return (rect.y1 - rect.y0) / 2.;
 	}
@@ -1322,7 +1335,7 @@ void Document::SetTheme (Theme *theme)
 		m_pView->UpdateTheme ();
 }
 
-bool Document::OnSignal (SignalId Signal, Object *Child)
+bool Document::OnSignal (SignalId Signal, G_GNUC_UNUSED Object *Child)
 {
 	if (Signal == OnThemeChangedSignal) {
 		m_BondLength = m_Theme->GetBondLength ();
@@ -1437,12 +1450,56 @@ bool Document::SetProperty (unsigned property, char const *value)
 		double length = strtod (value, &end);
 		if (*end != 0)
 			return false;
-		gcu::Document::m_Scale = m_Theme->GetBondLength () / length; 
+		gcu::Document::SetScale (m_Theme->GetBondLength () / length); 
+		break;
 	}
 	}
 	return true;
 }
 
+std::string Document::GetProperty (unsigned property) const
+{
+	ostringstream res;
+	switch (property) {
+	case GCU_PROP_DOC_FILENAME:
+		res << GetFileName ();
+		break;
+	case GCU_PROP_DOC_MIMETYPE:
+		res << m_FileType;
+		break;
+	case GCU_PROP_DOC_TITLE:
+		res << GetTitle ();
+		break;
+	case GCU_PROP_DOC_COMMENT:
+		res << m_comment;
+		break;
+	case GCU_PROP_DOC_CREATOR:
+		res << m_author;
+		break;
+	case GCU_PROP_DOC_CREATION_TIME: {
+		char buf[16];
+		*buf = 0;
+		g_date_strftime (buf, 16, "%F", &CreationDate);
+		res << buf;
+		break;
+	}
+	case GCU_PROP_DOC_MODIFICATION_TIME: {
+		char buf[16];
+		*buf = 0;
+		g_date_strftime (buf, 16, "%F", &RevisionDate);
+		res << buf;
+		break;
+	}
+	case GCU_PROP_THEME_BOND_LENGTH: {
+		res << m_Theme->GetBondLength ();
+		break;
+	default:
+		return gcu::Document::GetProperty (property);
+	}
+	}
+	return res.str ();
+}
+
 GtkWindow *Document::GetGtkWindow ()
 {
 	return (m_Window)? m_Window->GetWindow (): NULL;
