@@ -100,14 +100,12 @@ Inserts a text inside an existing instance during a paste operation.
 /*!
 @param node the xml node representing a portion of the text.
 @param pos where to insert the new text.
-@param level the imbrication level used to filter out extra text nodes
-added by libxml2.
 @param cur_size the current size in bytes.
 
 Loads a portion of a text with a unique set of attributes. \a pos and
 \a cur_size are updated.
 */
-	bool LoadNode (xmlNodePtr node, unsigned &pos, int level = 0, int cur_size = 0);
+	bool LoadNode (xmlNodePtr node, unsigned &pos, int cur_size = 0);
 /*!
 Used to add a representation of the text in the view.
 */
@@ -170,31 +168,13 @@ and GCU_PROP_TEXT_JUSTIFICATION.
 @return true if the property could be set, or if the property is not relevant, false otherwise.
 */
 	bool SetProperty (unsigned property, char const *value);
+/*!
+*/
+	void InterlineChanged (double interline);
+/*!
+*/
+	void JustificationChanged (GtkJustification justification);
 
-/*!\fn SetAlign(PangoAlignment Align)
-@param Align the new PangoAlignment for the text.
-
-Sets the PangoAlignment for the text. Default is PANGO_ALIGN_LEFT.
-*/
-/*!\fn GetAlign()
-@return the current PangoAlignment for the text instance.
-*/
-/*!fn GetRefAlign()
-@return the current PangoAlignment for the text instance as a reference.
-*/
-GCU_PROP (PangoAlignment, Align)
-/*!\fn SetJustified(bool Justified)
-@param Justified whether the text should be justified.
-
-Sets the justification for the text. Default is false.
-*/
-/*!\fn GetJustified()
-@return the justification for the text.
-*/
-/*!fn GetRefJustified()
-@return the justification for the text as a reference.
-*/
-GCU_PROP (bool, Justified)
 /*!\fn SetAnchor(gccv::Anchor Anchor)
 @param Anchor the new gccv::Anchor for the text.
 
@@ -209,6 +189,10 @@ a carbon atom symbol using the current document theme.
 @return the gccv::Anchor used for the text as a reference.
 */
 GCU_PROP (gccv::Anchor, Anchor)
+
+private:
+	double m_Interline;
+	GtkJustification m_Justification;
 };
 
 }	//	namespace gcp
