@@ -26,6 +26,7 @@
 #define GCHEMPAINT_TOOLS_H
 
 #include <gcu/dialog.h>
+#include <gcu/gtkcomboperiodic.h>
 #include <map>
 
 /*!\file*/
@@ -102,11 +103,6 @@ using the Mendeleiev table widget.
 */
 	void SetElement (int Z);
 /*!
-Called by the framework when the tearable property of the Mendeleiev table
-widget changed.
-*/
-	void Update (void);
-/*!
 Called by the framework when the Help button is clicked.
 */
 	void OnHelp ();
@@ -117,6 +113,7 @@ private:
 	std::map<Tool*, int> m_Pages;
 	GtkNotebook *m_Book;
 	Tool *m_Tool;
+	GtkComboPeriodic *m_Mendeleiev;
 };
 
 }	// namespace gcp
