@@ -28,7 +28,10 @@
 #include "crystalview.h"
 #include "cylinder.h"
 #include "matrix.h"
+#include "objprops.h"
+#include "spacegroup.h"
 #include "sphere.h"
+#include "transform3d.h"
 #include "vector.h"
 #include "xml-utils.h"
 #include <gtk/gtk.h>
@@ -36,15 +39,15 @@
 #include <libintl.h>
 #include <clocale>
 #include <cmath>
+#include <set>
 #include <vector>
 #include <GL/gl.h>
 #include <cstring>
+#include <sstream>
 
 #define __max(x,y)  ((x) > (y)) ? (x) : (y)
 #define __min(x,y)  ((x) < (y)) ? (x) : (y)
-#define PREC 1e-3
 
-using namespace OpenBabel;
 using namespace std;
 
 namespace gcu
@@ -67,12 +70,8 @@ gchar const *LatticeName[] = {
 	"triclinic"
 };
 
-#ifdef HAVE_OPENBABEL_2_2
 CrystalDoc::CrystalDoc (Application *App): GLDocument (App),
 	m_SpaceGroup (NULL)
-#else
-CrystalDoc::CrystalDoc (Application *App): GLDocument (App)
-#endif
 {
 }
 
@@ -180,7 +179,6 @@ void CrystalDoc::ParseXMLTree (xmlNode* xml)
 			if (i < 14)
 				m_lattice = (gcLattices)i;
 			xmlFree (txt);
-#ifdef HAVE_OPENBABEL_2_2
 		} else if (!strcmp ((gchar*) node->name, "group")) {
 			SpaceGroup *group = new SpaceGroup ();
 			txt = (char*) xmlGetProp (node, (xmlChar*) "Hall");
@@ -205,8 +203,8 @@ void CrystalDoc::ParseXMLTree (xmlNode* xml)
 				}
 				child = child->next;
 			}
-			m_SpaceGroup = group;
-#endif
+			m_SpaceGroup = SpaceGroup::Find (group);
+			delete group;
 		} else if (!strcmp ((gchar*) node->name, "cell")) {
 			txt = (char*) xmlGetProp (node, (xmlChar*)"a");
 			if (txt) {
@@ -250,7 +248,7 @@ void CrystalDoc::ParseXMLTree (xmlNode* xml)
 		} else if (!strcmp ((gchar*) node->name, "atom")) {
 			CrystalAtom *pAtom = CreateNewAtom ();
 			if (pAtom->Load (node))
-				AtomDef.push_back (pAtom);
+				AddChild (pAtom);
 			else
 				delete pAtom;
 		} else if (!strcmp ((gchar*) node->name, "line")) {
@@ -316,29 +314,29 @@ void CrystalDoc::Update()
 		Lines.pop_front();
 	}
 
+	// update space group
+	m_SpaceGroup = FindSpaceGroup ();
+
 	////////////////////////////////////////////////////////////
 	//Establish list of atoms
 	CrystalAtomList::iterator i, iend = AtomDef.end ();
-	
-#ifdef HAVE_OPENBABEL_2_2
+
 	if (m_SpaceGroup) {
-		vector3 v;
-		list<vector3> d;
+		Vector v;
+		list<Vector> d;
 		for (i = AtomDef.begin(); i != iend; i++) {
-			v.x () = (*i)->x ();
-			v.y () = (*i)->y ();
-			v.z () = (*i)->z ();
+			v.SetX ((*i)->x ());
+			v.SetY ((*i)->y ());
+			v.SetZ ((*i)->z ());
 			d = m_SpaceGroup->Transform (v);
-			list<vector3>::iterator vi, viend = d.end();
+			list<Vector>::iterator vi, viend = d.end();
 			CrystalAtom atom (**i);
 			for (vi=d.begin (); vi!= viend; vi++) {
-				atom.SetCoords ((*vi).x(), (*vi).y(), (*vi).z());
+				atom.SetCoords ((*vi).GetX(), (*vi).GetY(), (*vi).GetZ());
 				Duplicate (atom);
 			}
 		}
-	} else
-#endif
-	for (i = AtomDef.begin(); i != iend; i++) {
+	} else for (i = AtomDef.begin(); i != iend; i++) {
 		Duplicate(**i);
 		switch (m_lattice) {
 		case body_centered_cubic:
@@ -546,6 +544,7 @@ void CrystalDoc::Update()
 		m_MaxDist = __max (m_MaxDist, d);
 		(*j)->Move (- x, - y, - z);
 	}
+	m_SpaceGroup = FindSpaceGroup ();
 }
 
 void CrystalDoc::Duplicate (CrystalAtom& Atom)
@@ -595,8 +594,10 @@ void CrystalDoc::Draw (Matrix const &m) const
 	double red, green, blue, alpha;
 	for (i = Atoms.begin (); i != iend; i++)
 		if (!(*i)->IsCleaved ()) {
-			v = (*i)->GetVector ();
-			v = m * v;
+			v.SetZ ((*i)->x ());
+			v.SetX ((*i)->y ());
+			v.SetY ((*i)->z ());
+			v = m.glmult (v);
 			(*i)->GetColor (&red, &green, &blue, &alpha);
 			glColor4d (red, green, blue, alpha) ;
 			sp.draw (v, (*i)->r () * (*i)->GetEffectiveRadiusRatio ());
@@ -609,11 +610,11 @@ void CrystalDoc::Draw (Matrix const &m) const
 			v.SetZ ((*j)->X1 ());
 			v.SetX ((*j)->Y1 ());
 			v.SetY ((*j)->Z1 ());
-			v = m * v;
+			v = m.glmult (v);
 			v1.SetZ ((*j)->X2 ());
 			v1.SetX ((*j)->Y2 ());
 			v1.SetY ((*j)->Z2 ());
-			v1 = m * v1;
+			v1 = m.glmult (v1);
 			(*j)->GetColor (&red, &green, &blue, &alpha);
 			glColor4d (red, green, blue, alpha) ;
 			cyl.draw (v, v1, (*j)->GetRadius ());
@@ -669,7 +670,6 @@ xmlDocPtr CrystalDoc::BuildXMLTree () const
 		
 		node = xmlNewDocNode(xml, NULL, (xmlChar*)"lattice", (xmlChar*)LatticeName[m_lattice]);
 		if (node) xmlAddChild(xml->children, node); else throw (int) 0;
-#ifdef HAVE_OPENBABEL_2_2
 		if (m_SpaceGroup) {
 			node = xmlNewDocNode (xml, NULL, (xmlChar*) "group", NULL);
 			if (node)
@@ -685,18 +685,17 @@ xmlDocPtr CrystalDoc::BuildXMLTree () const
 					xmlNewProp (node, (xmlChar*) "HM", (xmlChar*) name.c_str ());
 			}
 			xmlNodePtr child;
-			transform3dIterator i;
-			transform3d const *t = m_SpaceGroup->BeginTransform (i);
+			list <Transform3d*>::const_iterator i;
+			Transform3d const *t = m_SpaceGroup->GetFirstTransform (i);
 			while (t) {
 				child = xmlNewDocNode (xml, NULL, (xmlChar*) "transform", (xmlChar const*) t->DescribeAsString ().c_str ());
 				if (child)
 					xmlAddChild (node, child);
 				else
 					throw (int) 0;
-				t = m_SpaceGroup->NextTransform (i);
+				t = m_SpaceGroup->GetNextTransform (i);
 			}
 		}
-#endif
 		node = xmlNewDocNode(xml, NULL, (xmlChar*)"cell", NULL);
 		if (node) xmlAddChild(xml->children, node); else throw (int) 0;
 		snprintf(buf, sizeof(buf), "%g", m_a);
@@ -761,113 +760,383 @@ xmlDocPtr CrystalDoc::BuildXMLTree () const
 	}
 }
 
-#ifdef HAVE_OPENBABEL_2_2
-bool CrystalDoc::ImportOB (OBMol &mol)
+bool CrystalDoc::Loaded () throw (LoaderError)
 {
-	OBUnitCell *cell = dynamic_cast<OBUnitCell*> (mol.GetData (OBGenericDataType::UnitCell));
-	if (cell == NULL)
-		return false;
-	m_a = cell->GetA () * 100;
-	m_b = cell->GetB () * 100;
-	m_c = cell->GetC () * 100;
-	m_alpha = cell ->GetAlpha ();
-	m_beta = cell->GetBeta ();
-	m_gamma = cell->GetGamma ();
-	string const group_name = cell->GetSpaceGroupName ();
-	m_SpaceGroup = cell->GetSpaceGroup ();
-    if (!m_SpaceGroup)
-		return false;
-	int lattice = cell->GetLatticeType ();
-	switch (lattice) {
-	case OBUnitCell::Triclinic:
-		m_lattice = triclinic;
-		break;
-	case OBUnitCell::Monoclinic:
-		switch (group_name[0]) {
-		case 'C':
-			m_lattice = base_centered_monoclinic;
-			break;
-		default:
-			m_lattice = monoclinic;
-			break;
-		}
-		break;
-	case OBUnitCell::Orthorhombic:
-		switch (group_name[0]) {
-		case 'C':
-			m_lattice = base_centered_orthorhombic;
-			break;
-		case 'I':
-			m_lattice = body_centered_orthorhombic;
-			break;
-		case 'F':
-			m_lattice = face_centered_orthorhombic;
-			break;
-		default:
-			m_lattice = orthorhombic;
-			break;
-		}
-		break;
-	case OBUnitCell::Tetragonal:
-		switch (group_name[0]) {
-		case 'I':
-			m_lattice = body_centered_tetragonal;
-			break;
-		default:
-			m_lattice = tetragonal;
-			break;
-		}
-		break;
-	case OBUnitCell::Rhombohedral:
-		m_lattice = rhombohedral;
-		break;
-	case OBUnitCell::Hexagonal:
-		m_lattice = hexagonal;
-		break;
-	case OBUnitCell::Cubic:
-		switch (group_name[0]) {
-		case 'I':
-			m_lattice = body_centered_cubic;
-			break;
-		case 'F':
-			m_lattice = face_centered_cubic;
-			break;
-		default:
-			m_lattice = cubic;
-			break;
-		}
-		break;
-	}
-	matrix3x3 m = cell->GetFractionalMatrix ();
-	vector3 v;
-	// now get the atoms
-	OBAtomIterator it;
-	OBAtom *atom = mol.BeginAtom (it);
-	CrystalAtom *catom;
+	if (m_NameCommon.length () > 0)
+		SetTitle (m_NameCommon);
+	else if (m_NameMineral.length () > 0)
+		SetTitle (m_NameMineral);
+	else if (m_NameSystematic.length () > 0)
+		SetTitle (m_NameSystematic);
+	else if (m_NameStructure.length () > 0)
+		SetTitle (m_NameStructure);
+	LineDef.push_back (new CrystalLine (edges, 0., 0., 0., 0., 0., 0., 10., .25 , .25, .25 , 1.));
+	// set radii
+	CrystalAtomList::iterator i, iend = AtomDef.end ();
 	GcuAtomicRadius radius;
 	radius.type = GCU_VAN_DER_WAALS;
 	radius.charge = 0;
     radius.cn = -1;
     radius.spin = GCU_N_A_SPIN;
     radius.scale = NULL;
-	while (atom) {
-		v.x () = atom->GetX();
-		v.y () = atom->GetY();
-		v.z () = atom->GetZ();
-		v *= m;
-		radius.Z = atom->GetAtomicNum ();
-		catom = new CrystalAtom (radius.Z, v.x (), v.y (), v.z ());
+	/* use van der Waals radii for now in all cases, using ionic radii would need the
+	evaluation of the coordination numbers */
+	for (i = AtomDef.begin (); i != iend; i++) {
+		radius.Z = (*i)->GetZ ();
 		if (gcu_element_get_radius (&radius)) {
-			catom->SetRadius (radius);
-			catom->SetEffectiveRadiusRatio (.4);
+			(*i)->SetRadius (radius);
+			(*i)->SetEffectiveRadiusRatio (.4);
 		}
-		AtomDef.push_back (catom);
-		atom = mol.NextAtom (it);
+		(*i)->SetDefaultColor ();
 	}
-	LineDef.push_back (new CrystalLine (edges, 0., 0., 0., 0., 0., 0., 10., .25 , .25, .25 , 1.));
+	switch (m_lattice) { // ensure that the paraleters are coherent
+	case cubic:
+	case body_centered_cubic:
+	case face_centered_cubic:
+		m_alpha = m_beta = m_gamma  = 90;
+		m_b = m_c = m_a;
+		break;
+	case hexagonal:
+		m_alpha = m_beta = 90;
+		m_gamma  = 120;
+		m_b = m_a;
+		break;
+	case tetragonal:
+	case body_centered_tetragonal:
+		m_alpha = m_beta = m_gamma  = 90;
+		m_b = m_a;
+		break;
+	case orthorhombic:
+	case base_centered_orthorhombic:
+	case body_centered_orthorhombic:
+	case face_centered_orthorhombic:
+		m_alpha = m_beta = m_gamma  = 90;
+		break;
+	case rhombohedral:
+		m_beta = m_gamma = m_alpha;
+		m_b = m_c = m_a;
+		break;
+	case monoclinic:
+	case base_centered_monoclinic:
+		m_alpha = m_gamma  = 90;
+		break;
+	case triclinic:
+		break;
+	}
+
 	Update ();
+	return false;	// no pending reference updated
+}
+
+bool CrystalDoc::SetProperty (unsigned property, char const *value)
+{
+	switch (property) {
+	case GCU_PROP_CELL_A:
+		m_a = strtod (value, NULL) * GetScale ();
+		break;
+	case GCU_PROP_CELL_B:
+		m_b = strtod (value, NULL) * GetScale ();
+		break;
+	case GCU_PROP_CELL_C:
+		m_c = strtod (value, NULL) * GetScale ();
+		break;
+	case GCU_PROP_CELL_ALPHA:
+		m_alpha = strtod (value, NULL);
+		break;
+	case GCU_PROP_CELL_BETA:
+		m_beta = strtod (value, NULL);
+		break;
+	case GCU_PROP_CELL_GAMMA:
+		m_gamma = strtod (value, NULL);
+		break;
+	case GCU_PROP_CHEMICAL_NAME_COMMON:
+		m_NameCommon = value;
+		break;
+	case GCU_PROP_CHEMICAL_NAME_SYSTEMATIC:
+		m_NameSystematic = value;
+		break;
+	case GCU_PROP_CHEMICAL_NAME_MINERAL:
+		m_NameMineral = value;
+		break;
+	case GCU_PROP_CHEMICAL_NAME_STRUCTURE:
+		m_NameStructure = value;
+		break;
+	case GCU_PROP_SPACE_GROUP: {
+		m_SpaceGroup = SpaceGroup::GetSpaceGroup (value);
+		char type = (*value == '-')? value[1]: value[0];
+		int id = m_SpaceGroup->GetId ();
+		if (id <= 2)
+			m_lattice = triclinic;
+		else if (id <= 15)
+			m_lattice = (type == 'P')? monoclinic: base_centered_monoclinic;
+		else if (id <= 74)
+			switch (type) {
+			case 'P':
+				m_lattice = orthorhombic;
+				break;
+			case 'I':
+				m_lattice = body_centered_orthorhombic;
+				break;
+			case 'F':
+				m_lattice = face_centered_orthorhombic;
+				break;
+			default:
+				m_lattice = base_centered_orthorhombic;
+				break;
+			}
+		else if (id <= 142)
+			m_lattice = (type == 'P')? tetragonal: body_centered_tetragonal;
+		else if (id <= 167)
+			m_lattice = rhombohedral;
+		else if (id <= 194)
+			m_lattice = hexagonal;
+		else {
+			switch (type) {
+			case 'P':
+				m_lattice = cubic;
+				break;
+			case 'I':
+				m_lattice = body_centered_cubic;
+				break;
+			case 'F':
+				m_lattice = face_centered_cubic;
+				break;
+			}
+		}
+		break;
+	}
+	default:
+		return false;
+	}
 	return true;
 }
-#endif
+
+std::string CrystalDoc::GetProperty (unsigned property) const
+{
+	ostringstream res;
+	switch (property) {
+	case GCU_PROP_DOC_TITLE:
+		return m_Title;
+	case GCU_PROP_CHEMICAL_NAME_COMMON:
+		return m_NameCommon;
+	case GCU_PROP_CHEMICAL_NAME_SYSTEMATIC:
+		return m_NameSystematic;
+	case GCU_PROP_CHEMICAL_NAME_MINERAL:
+		return m_NameMineral;
+	case GCU_PROP_CELL_A:
+		res << m_a / GetScale ();
+		break;
+	case GCU_PROP_CELL_B:
+		res << m_b / GetScale ();
+		break;
+	case GCU_PROP_CELL_C:
+		res << m_c / GetScale ();
+		break;
+	case GCU_PROP_CELL_ALPHA:
+		res << m_alpha;
+		break;
+	case GCU_PROP_CELL_BETA:
+		res << m_beta;
+		break;
+	case GCU_PROP_CELL_GAMMA:
+		res << m_gamma;
+		break;
+	case GCU_PROP_SPACE_GROUP:
+		return m_SpaceGroup->GetHallName ();
+	default:
+		return GLDocument::GetProperty (property);
+	}
+	return res.str ();
+}
+
+void CrystalDoc::AddChild (Object* object)
+{
+	Object::AddChild (object);
+	CrystalAtom *atom = dynamic_cast <CrystalAtom *> (object);
+	if (atom) {
+		AtomDef.remove (atom); // don't add more than needed (a set might be better than a list)
+		AtomDef.push_back (atom);
+	}
+}
+
+SpaceGroup const *CrystalDoc::FindSpaceGroup ()
+{
+	if (!AtomDef.size ())
+		return NULL;
+	if (m_SpaceGroup)
+		return (m_SpaceGroup);
+	unsigned start, end, id;
+	switch (m_lattice) {
+	case cubic:
+	case body_centered_cubic:
+	case face_centered_cubic:
+		start = 195;
+		end = 230;
+		break;
+	case hexagonal:
+		start = 168;
+		end = 194;
+		break;
+	case tetragonal:
+	case body_centered_tetragonal:
+		start = 75;
+		end = 142;
+		break;
+	case orthorhombic:
+	case base_centered_orthorhombic:
+	case body_centered_orthorhombic:
+	case face_centered_orthorhombic:
+		start = 16;
+		end = 74;
+		break;
+	case rhombohedral:
+		start = 143;
+		end = 167;
+		break;
+	case monoclinic:
+	case base_centered_monoclinic:
+		start = 3;
+		end = 15;
+		break;
+	case triclinic:
+		start = 1;
+		end = 2;
+		break;
+	default:
+		start = end = 0;
+	}
+	//make a list of all atoms inside the cell
+	list <CrystalAtom *> atoms;
+	CrystalAtom *a;
+	double x, y, z;
+	CrystalAtomList::iterator i, i0, iend = AtomDef.end ();
+	for (i = AtomDef.begin (); i != iend; i++) {
+		atoms.push_back (new CrystalAtom (**i));
+		switch (m_lattice) {
+		case body_centered_cubic:
+		case body_centered_tetragonal:
+		case body_centered_orthorhombic:
+			a = new CrystalAtom (**i);
+			a->GetCoords (&x, &y, &z);
+			x = (x > .5 - PREC)? x - .5: x + .5;
+			y = (y > .5 - PREC)? y - .5: y + .5;
+			z = (z > .5 - PREC)? z - .5: z + .5;
+			a->SetCoords (x, y, z);
+			atoms.push_back (a);
+			break;
+		case face_centered_cubic:
+		case face_centered_orthorhombic:
+			a = new CrystalAtom (**i);
+			a->GetCoords (&x, &y, &z);
+			x = (x > .5 - PREC)? x - .5: x + .5;
+			z = (z > .5 - PREC)? z - .5: z + .5;
+			a->SetCoords (x, y, z);
+			atoms.push_back (a);
+			a = new CrystalAtom (**i);
+			a->GetCoords (&x, &y, &z);
+			y = (y > .5 - PREC)? y - .5: y + .5;
+			z = (z > .5 - PREC)? z - .5: z + .5;
+			a->SetCoords (x, y, z);
+			atoms.push_back (a);
+		case base_centered_orthorhombic:
+		case base_centered_monoclinic:
+			a = new CrystalAtom (**i);
+			a->GetCoords (&x, &y, &z);
+			x = (x > .5 - PREC)? x - .5: x + .5;
+			y = (y > .5 - PREC)? y - .5: y + .5;
+			a->SetCoords (x, y, z);
+			atoms.push_back (a);
+			break;
+		default:
+			break;
+		}
+	}
+	iend = atoms.end ();
+	SpaceGroup const *res = NULL;
+	Vector v;
+	std::list <Vector>::iterator j, jend;
+	for (id = end; id >= start; id--) {
+		std::list <SpaceGroup const *> &groups = SpaceGroup::GetSpaceGroups (id);
+		std::list <SpaceGroup const *>::iterator g, gend = groups.end ();
+		for (g = groups.begin (); g != gend; g++) {
+			for (i = atoms.begin (); i != iend; i++) {
+				a = new CrystalAtom (**i);
+				v = a->GetVector ();
+				std::list <Vector> vv = (*g)->Transform (v);
+				jend = vv.end ();
+				for (j = vv.begin (); j != jend; j++) {
+					x = (*j).GetX ();
+					y = (*j).GetY ();
+					z = (*j).GetZ ();
+					while (x > 1. - PREC)
+						x -= 1.;
+					while (y > 1. - PREC)
+						y -= 1.;
+					while (z > 1. - PREC)
+						z -= 1.;
+					a->SetCoords (x, y, z);
+					for (i0 = atoms.begin (); i0 != iend; i0++)
+						if (*a == **i0)
+							break;
+					if (i0 == iend)
+						goto end_loop;
+				}
+				delete a;
+				if (j != jend)
+					goto end_loop;
+			}
+			if (i == iend)
+				break;
+end_loop:;
+		}
+		if (g != gend) {
+			res = *g;
+			break;
+		}
+	}
+	// clean atoms
+	for (i = atoms.begin (); i != iend; i++)
+		delete *i;
+	// now, search for duplicates in AtomDef
+	set <CrystalAtom *> dups;
+	iend = AtomDef.end ();
+	for (i = AtomDef.begin (); i != iend; i++) {
+		if (dups.find (*i) != dups.end ())
+			continue;
+		a = new CrystalAtom (**i);
+		v = a->GetVector ();
+		std::list <Vector> vv = res->Transform (v);
+		for (j = vv.begin (); j != jend; j++) {
+			x = (*j).GetX ();
+			y = (*j).GetY ();
+			z = (*j).GetZ ();
+			while (x > 1. - PREC)
+				x -= 1.;
+			while (y > 1. - PREC)
+				y -= 1.;
+			while (z > 1. - PREC)
+				z -= 1.;
+			a->SetCoords (x, y, z);
+			for (i0 = i, i0++; i0 != iend; i0++) {
+				if (dups.find (*i0) != dups.end ())
+					continue;
+				if (*a == **i0) {
+					if ((*i0)->x () < (*i)->x () || (*i0)->y () < (*i)->y () || (*i0)->z () < (*i)->z ()) {
+						dups.insert (*i);
+						goto end_loop1;
+					}
+					dups.insert (*i0);
+				}
+			}
+		}
+end_loop1:;
+	}
+	set <CrystalAtom *>::iterator k, kend = dups.end ();
+	for (k = dups.begin (); k != kend; k++) {
+		AtomDef.remove (*k);
+		delete *k;
+	}
+	return res;
+}
 
 }	//	namespace gcu
