@@ -46,10 +46,7 @@ struct _GtkCrystalViewerClass
 static void on_size (GtkCrystalViewer* w, GtkAllocation *allocation, G_GNUC_UNUSED gpointer user_data)
 {
 	GtkWidget *widget = gtk_bin_get_child (GTK_BIN (w));
-	bool visible = false;
-	if (widget)
-		g_object_get (G_OBJECT (widget), "visible", &visible, NULL);
-	if (visible)
+	if (widget && gtk_widget_get_visible (widget))
 		gtk_widget_size_allocate (widget, allocation);
 }
 
