@@ -65,7 +65,9 @@ Object::~Object ()
 {
 	if (m_Id) {
 		if (m_Parent) {
-			GetDocument ()->m_DirtyObjects.erase (this);
+			Document *doc = GetDocument ();
+			if (doc)
+				doc->m_DirtyObjects.erase (this);
 			m_Parent->m_Children.erase (m_Id);
 		}
 		g_free (m_Id);
@@ -81,6 +83,8 @@ Object::~Object ()
 			m_Children.erase ((*i).first);
 		}
 	}
+	while (!m_Links.empty ())
+		Unlink (*(m_Links.begin ()));
 }
 
 void Object::Clear ()
@@ -192,8 +196,12 @@ void Object::SetParent (Object* Parent)
 	if (Parent)
 		Parent->AddChild(this);
 	else {
-		if (m_Parent)
-			m_Parent->m_Children.erase (m_Id); 
+		if (m_Parent) {
+			Document *doc = GetDocument ();
+			if (doc)
+				doc->m_DirtyObjects.erase (this);
+			m_Parent->m_Children.erase (m_Id);
+		}
 		m_Parent = NULL;
 	}
 }
@@ -587,13 +595,18 @@ Object* Object::GetNextLink (set<Object*>::iterator& i)
 	return *i;
 }
 
+void Object::Link (Object *object)
+{
+	m_Links.insert (object);
+}
+
 void Object::Unlink (Object *object)
 {
 	m_Links.erase (object);
-	OnUnlink (object);
+	object->OnUnlink (this);
 }
 
-void Object::OnUnlink (G_GNUC_UNUSED G_GNUC_UNUSED Object *object)
+void Object::OnUnlink (G_GNUC_UNUSED Object *object)
 {
 }
 
