@@ -49,6 +49,9 @@ typedef struct {
 	char **args;
 	pthread_t thread;*/
 	NPP instance;
+
+	NPStream *m_Stream;
+	bool m_ExpectingStream;
 } ChemPlugin;
 
 static NPError ChemNew (NPMIMEType mime_type, NPP instance,
@@ -110,14 +113,18 @@ static NPError ChemNew (NPMIMEType mime_type, NPP instance,
 	write (to_pipe, mime_type, strlen ((char*) mime_type));
 	write (to_pipe, "\n", 1);
 	i = 0;
-	while (strcmp (argn[i++], "PARAM"));
+	while (i < argc && strcmp (argn[i++], "PARAM"));
 	for (; i < argc; i++) {
+		if (argn[i] == NULL || argv[i] == NULL)
+			break;
 		write (to_pipe, argn[i], strlen (argn[i]));
 		write (to_pipe, "\n", 1);
 		write (to_pipe, argv[i], strlen (argv[i]));
 		write (to_pipe, "\n", 1);
 	}
 	write (to_pipe, "end\n", 4);
+	plugin->m_Stream = NULL;
+	plugin->m_ExpectingStream = true;
 	return NPERR_NO_ERROR;
 }
 
@@ -160,20 +167,40 @@ static NPError ChemNewStream (NPP instance, G_GNUC_UNUSED NPMIMEType type, G_GNU
 		      G_GNUC_UNUSED NPBool seekable, uint16 *stype)
 #endif
 {
+	if (!stream || !stream->url)
+		return NPERR_GENERIC_ERROR;
+
 	ChemPlugin *plugin;
 
 	if (instance == NULL)
 		return NPERR_INVALID_INSTANCE_ERROR;
-	
+
 	plugin = (ChemPlugin *) instance->pdata;
 	if (plugin == NULL)
-		return NPERR_NO_ERROR;	
-	
+		return NPERR_INVALID_INSTANCE_ERROR;	
+
+	if (plugin->m_Stream || !plugin->m_ExpectingStream)
+		return mozilla_funcs.destroystream (instance, stream, NPRES_DONE);
+
+	plugin->m_ExpectingStream = false;
+	plugin->m_Stream = stream;
+
 	*stype = NP_ASFILEONLY;
 
 	return NPERR_NO_ERROR;
 }
 
+static NPError ChemDestroyStream (G_GNUC_UNUSED NPP instance, G_GNUC_UNUSED NPStream* stream, G_GNUC_UNUSED NPReason reason)
+{
+	if (instance == NULL)
+		return NPERR_INVALID_INSTANCE_ERROR;
+	ChemPlugin *plugin = (ChemPlugin *) instance->pdata;
+	if (!plugin->m_Stream || plugin->m_Stream != stream)
+		return NPERR_GENERIC_ERROR;
+	plugin->m_Stream = NULL;
+	return NPERR_NO_ERROR;
+}
+
 static void ChemPrint (G_GNUC_UNUSED NPP instance, G_GNUC_UNUSED NPPrint *platformPrint)
 {
 // TODO: implement !!!
@@ -213,7 +240,7 @@ char *NP_GetMIMEDescription (void)
 			"chemical/x-pdb:pdb,ent:Protein DataBank;"
 			"chemical/x-jcamp-dx:dx,jdx:JCAMP Spectroscopic Data Exchange Format;"
 			"application/x-gcrystal:gcrystal:Crystalline structure model;"
-		/*	"chemical/x-cif:cif:Crystallographic Information File;"*/
+			"chemical/x-cif:cif:Crystallographic Information File;"
 			"application/x-gchempaint:gchempaint:2D chemical structures"
 			"chemical/x-cdx:cdx:ChemDraw binary format;"
 			"chemical/x-cdxml:cdxml:ChemDraw CDXML format;");
@@ -233,7 +260,6 @@ NPError NP_Initialize(NPNetscapeFuncs *mozFuncs, NPPluginFuncs *pluginFuncs) {
 		return NPERR_INVALID_FUNCTABLE_ERROR;
 
 	memcpy (&mozilla_funcs, mozFuncs, sizeof (NPNetscapeFuncs));
-	
 	pluginFuncs->version    = (NP_VERSION_MAJOR << 8) + NP_VERSION_MINOR;
 	pluginFuncs->size       = sizeof (NPPluginFuncs);
 #ifdef HAVE_NPFUNCTIONS_H
@@ -241,16 +267,14 @@ NPError NP_Initialize(NPNetscapeFuncs *mozFuncs, NPPluginFuncs *pluginFuncs) {
 	pluginFuncs->destroy    = (NPP_DestroyProcPtr) ChemDestroy;
 	pluginFuncs->setwindow  = (NPP_SetWindowProcPtr) ChemSetWindow;
 	pluginFuncs->newstream  = (NPP_NewStreamProcPtr) ChemNewStream;
+	pluginFuncs->destroystream  = (NPP_DestroyStreamProcPtr) ChemDestroyStream;
+	pluginFuncs->asfile     = (NPP_StreamAsFileProcPtr) ChemStreamAsFile;
 #else
 	pluginFuncs->newp       = NewNPP_NewProc (ChemNew);
 	pluginFuncs->destroy    = NewNPP_DestroyProc (ChemDestroy);
 	pluginFuncs->setwindow  = NewNPP_SetWindowProc (ChemSetWindow);
 	pluginFuncs->newstream  = NewNPP_NewStreamProc (ChemNewStream);
-#endif
-	pluginFuncs->destroystream = NULL;
-#ifdef HAVE_NPFUNCTIONS_H
-	pluginFuncs->asfile     = (NPP_StreamAsFileProcPtr) ChemStreamAsFile;
-#else
+	pluginFuncs->destroystream  = NewNPP_DestroyStreamProcPtr (ChemDestroyStream);
 	pluginFuncs->asfile     = NewNPP_StreamAsFileProc (ChemStreamAsFile);
 #endif
 	pluginFuncs->writeready = NULL;
