@@ -4,7 +4,7 @@
  * GChemPaint bonds plugin
  * chaintool.cc
  *
- * Copyright (C) 2006-2008 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2006-2009 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -35,6 +35,7 @@
 #include <gccv/canvas.h>
 #include <gccv/group.h>
 #include <gccv/line.h>
+#include <gcu/ui-builder.h>
 #include <gdk/gdkkeysyms.h>
 #include <glib/gi18n-lib.h>
 #include <cmath>
@@ -55,7 +56,7 @@ gcpChainTool::gcpChainTool (gcp::Application *App): gcp::Tool (App, "Chain")
 
 gcpChainTool::~gcpChainTool()
 {
-	delete m_Points;
+	delete [] m_Points;
 }
 	
 bool gcpChainTool::OnClicked()
@@ -345,7 +346,7 @@ static void on_number_changed (GtkSpinButton *btn, gcpChainTool *tool)
 
 void gcpChainTool::SetAngle (double angle)
 {
-	m_pView->GetDoc ()->SetBondAngle (angle);
+	m_pApp->GetActiveDocument ()->SetBondAngle (angle);
 }
 
 void gcpChainTool::SetLength (double length)
@@ -355,20 +356,22 @@ void gcpChainTool::SetLength (double length)
 
 GtkWidget *gcpChainTool::GetPropertyPage ()
 {
-	GladeXML *xml = glade_xml_new (GLADEDIR"/chain.glade", "chain", GETTEXT_PACKAGE);
-	m_LengthBtn = GTK_SPIN_BUTTON (glade_xml_get_widget (xml, "bond-length"));
+	gcu::UIBuilder *builder = new gcu::UIBuilder (UIDIR"/chain.ui", GETTEXT_PACKAGE);
+	m_LengthBtn = GTK_SPIN_BUTTON (builder->GetWidget ("bond-length"));
 	g_signal_connect (m_LengthBtn, "value-changed", G_CALLBACK (on_length_changed), this);
-	m_AngleBtn = GTK_SPIN_BUTTON (glade_xml_get_widget (xml, "bond-angle"));
+	m_AngleBtn = GTK_SPIN_BUTTON (builder->GetWidget ("bond-angle"));
 	g_signal_connect (m_AngleBtn, "value-changed", G_CALLBACK (on_angle_changed), this);
-	m_MergeBtn = GTK_TOGGLE_BUTTON (glade_xml_get_widget (xml, "merge"));
+	m_MergeBtn = GTK_TOGGLE_BUTTON (builder->GetWidget ("merge"));
 	g_signal_connect (m_MergeBtn, "toggled", G_CALLBACK (on_merge_toggled), NULL);
-	m_NumberBtn = GTK_SPIN_BUTTON (glade_xml_get_widget (xml, "bonds-number"));
+	m_NumberBtn = GTK_SPIN_BUTTON (builder->GetWidget ("bonds-number"));
 	gtk_widget_set_sensitive (GTK_WIDGET (m_NumberBtn), false);
 	g_signal_connect (m_NumberBtn, "value-changed", G_CALLBACK (on_number_changed), this);
-	m_AutoBtn = GTK_TOGGLE_BUTTON (glade_xml_get_widget (xml, "auto-number"));
+	m_AutoBtn = GTK_TOGGLE_BUTTON (builder->GetWidget ("auto-number"));
 	gtk_toggle_button_set_active (m_AutoBtn, true);
 	g_signal_connect (m_AutoBtn, "toggled", G_CALLBACK (on_number_toggled), this);
-	return glade_xml_get_widget (xml, "chain");
+	GtkWidget *res = builder->GetRefdWidget ("chain");
+	delete builder;
+	return res;
 }
 
 void gcpChainTool::Activate ()
@@ -448,7 +451,7 @@ bool gcpChainTool::CheckIfAllowed ()
 	return true;
 }
 
-bool gcpChainTool::OnEvent (GdkEvent* event)
+bool gcpChainTool::OnKeyPress (GdkEvent* event)
 {
 	if (event->type == GDK_KEY_PRESS) {
 		unsigned n;
