@@ -37,6 +37,7 @@
 #include <gccv/structs.h>
 #include <gccv/text.h>
 #include <gdk/gdkkeysyms.h>
+#include <glib/gi18n-lib.h>
 
 extern xmlDocPtr pXmlDoc;
 
@@ -48,6 +49,7 @@ gcpFragmentTool::gcpFragmentTool (gcp::Application *App): gcpTextTool (App, "Fra
 	m_ImContext = gtk_im_multicontext_new ();
 	g_signal_connect (G_OBJECT (m_ImContext), "commit",
 		G_CALLBACK (OnCommit), this);
+	m_OwnStatus = true;
 }
 
 gcpFragmentTool::~gcpFragmentTool ()
@@ -82,11 +84,10 @@ bool gcpFragmentTool::OnClicked ()
 	} else {
 		m_Fragment = NULL;
 	}
-/*	gccv::TextSelBounds bounds;
-	bool need_update = false;*/
+	unsigned start, end;
 	if (m_pObject) {
 		switch (m_pObject->GetType ()) {
-/*			case AtomType: {
+			case AtomType: {
 				gcp::Atom* pAtom = (gcp::Atom*) m_pObject;
 				if (pAtom->GetTotalBondsNumber () > 1)
 					return false;
@@ -95,8 +96,8 @@ bool gcpFragmentTool::OnClicked ()
 				gcp::Molecule *pMol = (gcp::Molecule*) pAtom->GetMolecule ();
 				map<Atom*, Bond*>::iterator i;
 				gcp::Bond *pBond = (gcp::Bond*) pAtom->GetFirstBond (i);
-				pFragment = new gcp::Fragment (x, y);
-				gcp::Atom* pFragAtom = (gcp::Atom*) pFragment->GetAtom ();
+				m_Fragment = new gcp::Fragment (x, y);
+				gcp::Atom* pFragAtom = (gcp::Atom*) m_Fragment->GetAtom ();
 				map<string, Object*>::iterator ie;
 				Object* electron = pAtom->GetFirstChild (ie);
 				while (electron) {
@@ -106,8 +107,8 @@ bool gcpFragmentTool::OnClicked ()
 				}
 				pMol->Remove (pAtom);
 				pAtom->SetParent (NULL);
-				pMol->AddFragment (pFragment);
-				pDoc->AddFragment (pFragment);
+				pMol->AddFragment (m_Fragment);
+				pDoc->AddFragment (m_Fragment);
 				pDoc->AbortOperation ();
 				gcp::Operation* pOp = pDoc->GetNewOperation (gcp::GCP_MODIFY_OPERATION);
 				pOp->AddObject (pAtom, 0);
@@ -116,34 +117,35 @@ bool gcpFragmentTool::OnClicked ()
 				m_pView->Remove (pAtom);
 				pFragAtom->SetZ (pAtom->GetZ ());
 				pFragAtom->SetId ((gchar*) pAtom->GetId ());
+				m_Fragment->OnChanged (false);
 				int n = pAtom->GetAttachedHydrogens ();
 				if (n) {
-					char* buf;
+					ostringstream stream;
+					stream << "H";
 					if (n > 1)
-						buf = g_strdup_printf ("H%d", n);
-					else
-						buf = g_strdup ("H");
-					bounds.start = bounds.cur = ((pAtom->GetBestSide ())? strlen (pAtom->GetSymbol ()): 0);
-					pFragment->OnSelChanged (&bounds);
-					gcp_pango_layout_replace_text (pFragment->GetLayout (),
-						bounds.cur,
-						0, buf, pDoc->GetPangoAttrList ());
-					bounds.cur +=  strlen (buf);
-					need_update = true;
-					g_free (buf);
+						stream << n;
+					string buf = stream.str ();
+					start = ((pAtom->GetBestSide ())? strlen (pAtom->GetSymbol ()): 0);
+					m_Fragment->GetTextItem ()->SetSelectionBounds (start, start);
+					m_Fragment->GetTextItem ()->ReplaceText (buf, start, 0);
+					end = start + buf.length ();
+					m_Fragment->OnChanged (false);
+					m_Fragment->AnalContent (start, end);
+					m_Fragment->GetTextItem ()->SetSelectionBounds (start, end);
 				}
 				delete pAtom;
 				if (pBond) {
 					pBond->ReplaceAtom (pAtom, pFragAtom);
 					pFragAtom->AddBond (pBond);
 					pOp->AddObject (pBond, 1);
+					pBond->SetDirty (); // force redraw
 				}
-				pOp->AddObject (pFragment, 1);
+				pOp->AddObject (m_Fragment, 1);
 				pDoc->FinishOperation ();
 				pDoc->EmptyTranslationTable ();
-				m_pObject = pFragment;
+				m_pObject = m_Fragment;
 				break;
-			}*/
+			}
 			case BondType:
 				return false;
 			case FragmentType:
@@ -154,12 +156,8 @@ bool gcpFragmentTool::OnClicked ()
 		if (!m_Fragment)
 			m_Fragment = static_cast <gcp::Fragment *> (m_pObject);
 		m_Fragment->SetSelected (gcp::SelStateUpdating);
+		m_Fragment->SetMode (m_CurMode = gcp::Fragment::AutoMode);
 		m_Active = m_Fragment->GetTextItem ();
-/*		if (need_update) {
-			m_Active->SetSelectionBounds (m_Active,  cur,  cur);
-			pFragment->AnalContent (start, cur);
-			pFragment->OnChanged (false);
-		}*/
 		m_pView->SetTextActive (m_Active);
 		m_Active->SetEditing (true);
 		m_Active->OnButtonPressed (m_x0, m_y0);
@@ -169,6 +167,7 @@ bool gcpFragmentTool::OnClicked ()
 		m_Fragment->SetEditor (this);
 	}
 	BuildTagsList ();
+	SetStatusText (gcp::Fragment::AutoMode);
 	return true;
 }
 
@@ -203,30 +202,42 @@ bool gcpFragmentTool::OnKeyPress (GdkEventKey *event)
 					// enter/quit charge mode
 					m_CurMode = (m_CurMode == gcp::Fragment::ChargeMode)? gcp::Fragment::AutoMode: gcp::Fragment::ChargeMode;
 					m_Fragment->SetMode (m_CurMode);
+					SetStatusText (m_CurMode);
 					BuildTagsList ();
 					break;
 				case GDK_underscore:
 					// enter/quit subscript (not stoichiometric) mode
 					m_CurMode = (m_CurMode == gcp::Fragment::SubscriptMode)? gcp::Fragment::AutoMode: gcp::Fragment::SubscriptMode;
 					m_Fragment->SetMode (m_CurMode);
+					SetStatusText (m_CurMode);
 					BuildTagsList ();
 					break;
 				case GDK_dead_circumflex:
 					// enter/quit superscript (not charge) mode
 					m_CurMode = (m_CurMode == gcp::Fragment::SuperscriptMode)? gcp::Fragment::AutoMode: gcp::Fragment::SuperscriptMode;
 					m_Fragment->SetMode (m_CurMode);
+					SetStatusText (m_CurMode);
 					BuildTagsList ();
 					break;
 				case GDK_n:
 					// enter/quit stoichiometry mode
 					m_CurMode = (m_CurMode == gcp::Fragment::StoichiometryMode)? gcp::Fragment::AutoMode: gcp::Fragment::StoichiometryMode;
 					m_Fragment->SetMode (m_CurMode);
+					SetStatusText (m_CurMode);
 					BuildTagsList ();
 					break;
 				case GDK_space:
 					// back to auto mode
 					m_CurMode = gcp::Fragment::AutoMode;
 					m_Fragment->SetMode (m_CurMode);
+					SetStatusText (m_CurMode);
+					BuildTagsList ();
+					break;
+				case GDK_equal:
+					// enter/quit normal mode
+					m_CurMode = (m_CurMode == gcp::Fragment::NormalMode)? gcp::Fragment::AutoMode: gcp::Fragment::NormalMode;
+					m_Fragment->SetMode (m_CurMode);
+					SetStatusText (m_CurMode);
 					BuildTagsList ();
 					break;
 				case GDK_z:
@@ -251,8 +262,6 @@ bool gcpFragmentTool::OnKeyPress (GdkEventKey *event)
 		if (event->keyval == GDK_KP_Enter || event->keyval == GDK_Return ||
 							event->keyval == GDK_space) // not allowed in fragments
 			return true;
-		if (gtk_im_context_filter_keypress (m_ImContext, event))
-			return true;
 		m_Active->OnKeyPressed (event);
 		return true;
 	}
@@ -298,21 +307,55 @@ bool gcpFragmentTool::CutSelection (GtkClipboard *clipboard)
 }
 
 /**
-* This method does nothing and always return false because pasting something inside
+* Accept only unformated text since pasting something else inside
 * a fragment is somewhat unsafe and will not be implemented in a foreseable future.
 */
-bool gcpFragmentTool::OnReceive (G_GNUC_UNUSED GtkClipboard *clipboard, G_GNUC_UNUSED GtkSelectionData *data, G_GNUC_UNUSED int type)
+bool gcpFragmentTool::OnReceive (GtkClipboard *clipboard, GtkSelectionData *selection_data, G_GNUC_UNUSED int type)
 {
-	return false;
+	if (!m_Active)
+		return false;
+	guint *DataType = (clipboard == gtk_clipboard_get (GDK_SELECTION_CLIPBOARD))? &gcp::ClipboardDataType: &gcp::ClipboardDataType1;
+	g_return_val_if_fail ((gtk_selection_data_get_target (selection_data) == gdk_atom_intern (gcp::targets[*DataType].target, FALSE)), FALSE);
+	int length = gtk_selection_data_get_length (selection_data);
+	char const *data = reinterpret_cast <char const *> (gtk_selection_data_get_data (selection_data));
+	gcp::Fragment *fragment = dynamic_cast <gcp::Fragment*> (m_Active->GetClient ());
+	unsigned start, end;
+	fragment->GetSelectionBounds (start, end);
+	switch (*DataType) {
+		case gcp::GCP_CLIPBOARD_UTF8_STRING: {
+			string s (data);
+			m_Active->ReplaceText (s, static_cast <int> (start), start - end);
+			break;
+		}
+		case gcp::GCP_CLIPBOARD_STRING: {
+			if (!g_utf8_validate (data, length, NULL)) {
+				gsize r, w;
+				char* newstr = g_locale_to_utf8 (data, length, &r, &w, NULL);
+				string s (newstr);
+				m_Active->ReplaceText (s, static_cast <int> (start), start - end);
+				g_free (newstr);
+			} else {
+				string s (data);
+				m_Active->ReplaceText (s, static_cast <int> (start), start - end);
+			}
+			break;
+		}
+	}
+	fragment->OnChanged (true);
+	return true;
 }
 
 bool gcpFragmentTool::Unselect ()
 {
 	if (!m_Active)
 		return true;
-	gcp::Fragment *fragment = dynamic_cast <gcp::Fragment*> (m_Active->GetClient ());;
-	if (fragment->Validate ())
-		return gcpTextTool::Unselect ();
+	gcp::Fragment *fragment = dynamic_cast <gcp::Fragment*> (m_Active->GetClient ());
+	if (fragment->Validate ()) {
+		bool result = gcpTextTool::Unselect ();
+		if (result)
+			m_pApp->ClearStatus ();
+		return result;
+	}
 	return false;
 }
 
@@ -379,8 +422,6 @@ void gcpFragmentTool::OnCommit (G_GNUC_UNUSED GtkIMContext *context, const gchar
 }
 
 void gcpFragmentTool::UpdateTagsList () {
-	// FIXME: write this
-	gccv::TextTagList *l = new gccv::TextTagList ();
 	if (m_Active) {
 		list <gccv::TextTag *> const *cur = m_Active->GetTags ();
 		unsigned start, end;
@@ -388,7 +429,6 @@ void gcpFragmentTool::UpdateTagsList () {
 		list <gccv::TextTag *>::const_iterator i, iend = cur->end ();
 		for (i = cur->begin (); i != iend; i++)
 			if ((*i)->GetStartIndex () < end && (*i)->GetEndIndex () >= end) {
-				l->push_front ((*i)->Duplicate ());
 				gccv::Tag tag = (*i)->GetTag ();
 				if (tag == gccv::Position) {
 					bool stacked;
@@ -398,18 +438,42 @@ void gcpFragmentTool::UpdateTagsList () {
 						break;
 					case gccv::Subscript:
 						m_CurMode = gcp::Fragment::SubscriptMode;
+						SetStatusText (m_CurMode);
 						break;
 					case gccv::Superscript:
 						m_CurMode = gcp::Fragment::SuperscriptMode;
+						SetStatusText (m_CurMode);
 						break;
 					}
-				} else if (tag == gcp::ChargeTag)
-					m_CurMode = gcp::Fragment::ChargeMode;
-				else if (tag == gcp::StoichiometryTag)
-					m_CurMode = gcp::Fragment::StoichiometryMode;
+				}
 			}
 		m_Fragment->SetMode (m_CurMode);
-		m_Active->SetCurTagList (l); // no tag
+		BuildTagsList ();
 	}
 }
 
+void gcpFragmentTool::SetStatusText (gcp::Fragment::FragmentMode mode)
+{
+	string status = _("Mode: ");
+	switch (mode) {
+	case gcp::Fragment::AutoMode:
+		status += _("auto");
+		break;
+	case gcp::Fragment::NormalMode:
+		status += _("normal");
+		break;
+	case gcp::Fragment::SubscriptMode:
+		status += _("subscript");
+		break;
+	case gcp::Fragment::SuperscriptMode:
+		status += _("superscript");
+		break;
+	case gcp::Fragment::ChargeMode:
+		status += _("charge");
+		break;
+	case gcp::Fragment::StoichiometryMode:
+		status += _("stoichiometry");
+		break;
+	}
+	m_pApp->SetStatusText (status.c_str ());
+}
