@@ -27,12 +27,9 @@
 #include "document.h"
 #include <glib.h>
 #include <glib/gi18n.h>
-#include <gtk/gtk.h>
 #include <gtk/gtkglinit.h>
 #include <gcu/chemistry.h>
-#include <goffice/goffice.h>
-#include <goffice/utils/go-file.h>
-#include <gio/gio.h>
+#include <gcu/element.h>
 #include <cstring>
 #include <cstdio>
 
@@ -89,6 +86,7 @@ int main(int argc, char *argv[])
 	textdomain (GETTEXT_PACKAGE);
 	gtk_init (&argc, &argv);
 	gtk_gl_init (&argc, &argv);
+	Element::LoadBODR ();
 
 	if (argc > 1 && argv[1][0] == '-') {
 		context = g_option_context_new (_(" [file]"));
